package main

import (
	"converge/pkg/converge"
	"converge/pkg/websocketutil"
	"fmt"
	"log"
	"net"
	"net/http"
	"os"
	"regexp"
)

func parsePublicId(path string) (publicId string, _ error) {
	pattern := regexp.MustCompile("^/[^/]+/([^/]+)$")
	matches := pattern.FindStringSubmatch(path)
	if len(matches) != 2 {
		return "", fmt.Errorf("Invalid URL path '%s'", path)
	}
	return matches[1], nil
}

func catchAllHandler(w http.ResponseWriter, r *http.Request) {
	http.Redirect(w, r, "/docs", http.StatusFound)
	return
}

func main() {

	downloadDir := "downloads"
	if len(os.Args) == 2 {
		downloadDir = os.Args[1]
	}
	log.Println("Doanload directory", downloadDir)

	admin := converge.NewAdmin()
	registrationService := websocketutil.WebSocketService{
		Handler: func(w http.ResponseWriter, r *http.Request, conn net.Conn) {
			publicId, err := parsePublicId(r.URL.Path)
			if err != nil {
				log.Printf("Cannot parse public id from url: '%v'\n", err)
				return
			}
			log.Printf("Got registration connection: '%s'\n", publicId)
			err = admin.Register(publicId, conn)
			if err != nil {
				log.Printf("Error %v\n", err)
			}
		},
	}
	clientService := websocketutil.WebSocketService{
		Handler: func(w http.ResponseWriter, r *http.Request, conn net.Conn) {
			publicId, err := parsePublicId(r.URL.Path)
			if err != nil {
				log.Printf("Cannot parse public id from url: '%v'\n", err)
				return
			}
			log.Printf("Got client connection: '%s'\n", publicId)
			err = admin.Connect(publicId, conn)
			if err != nil {
				log.Printf("Error %v\n", err)
			}
		},
	}

	// websocket endpoints
	http.HandleFunc("/agent/", registrationService.Handle)
	http.HandleFunc("/client/", clientService.Handle)

	// create filehandler with templating for html files.
	fileHandler := NewFileHandler(downloadDir)
	http.Handle("/docs/", http.StripPrefix("/docs/", fileHandler))
	http.HandleFunc("/", catchAllHandler)

	// Start HTTP server
	fmt.Println("Rendez-vous server listening on :8000")
	log.Fatal(http.ListenAndServe(":8000", nil))
}
